import json

from qgis.PyQt.QtCore import QObject

from qgis.core import (
    QgsVectorLayer,
)
from dataclasses import dataclass

from typing import List


@dataclass
class InterpolationArea:
    """InterpolationArea data class, used for partial interpolation of a reach"""

    from_profil_id: int = 0
    to_profil_id: int = 0
    longitudinal_step: float = 1.0

    def toDict(self):
        dict_ = self.__dict__.copy()
        return dict_

    @classmethod
    def fromDict(cls, dict_):
        return cls(**dict_)

    def copy(self):
        return InterpolationArea.fromDict(self.toDict())


class InterpolationAreaManager(QObject):
    """InterpolationArea manager class allowing to read and write interpolation_areas
    custom property of a layer"""

    INTERPOLATION_AREAS_KEY = "interpolation_areas"

    @classmethod
    def readInterpolationAreasFromLayer(
        cls, layer: QgsVectorLayer
    ) -> List[InterpolationArea]:
        if layer is None:
            return []

        str_value = layer.customProperty(cls.INTERPOLATION_AREAS_KEY)

        interpolation_areas = []
        if str_value:
            for dict_ in json.loads(str_value):
                interpolation_areas.append(InterpolationArea.fromDict(dict_))
        return interpolation_areas

    @classmethod
    def writeInterpolationAreasOfLayer(
        cls, layer: QgsVectorLayer, interpolation_areas: List[InterpolationArea]
    ) -> bool:
        if layer is None:
            return False

        # convert List[List[float]] to List[InterpolationArea]
        layer.setCustomProperty(
            cls.INTERPOLATION_AREAS_KEY,
            json.dumps([ia.toDict() for ia in interpolation_areas]),
        )

        return True
